package org.me4se.impl.jam;

import java.io.*;
import java.net.*;
import java.util.*;
import javax.microedition.midlet.*;
import org.me4se.impl.*;

/**
 * Manages the database of installed MIDlet suites. This is a model class (in
 * MVC). Create an instance with a proper base path and call the public methods
 * do install suites, remove or query them.
 * 
 * @author joerg@pleumann.de
 */
public class MIDletDatabase {

	/**
	 * Points to the database base path. Every suite resides in its
	 * own directory directly below this path.
	 */
	private String path;

	/**
	 * Holds the JAD files of all installed suites (as instances of
	 * JadFile).
	 */
	private Vector suites = new Vector();

	/**
	 * Creates an instance of the database.
	 */
	public MIDletDatabase(String path) {
		this.path = path;
		refresh();
	}

	/**
	 * Refreshes the list of installed suites.
	 */
	public void refresh() {
		suites.removeAllElements();

		File dir = new File(path);
		String[] names = dir.list();

		if (names != null) {
			for (int i = 0; i < names.length; i++) {
				String jadName = path + File.separatorChar + names[i] + File.separatorChar + names[i] + ".jad";
				JadFile jadFile = new JadFile();

				try {
					jadFile.load(jadName);
					suites.addElement(jadFile);
				}
				catch (IOException ignored) {
				}
			}
		}
	}

	/**
	 * Queries the number of installed suites.
	 */
	public int getSuiteCount() {
		return suites.size();
	}

	/**
	 * Queries the JAD file contents of the given suite.
	 */
	public JadFile getSuite(int index) {
		return (JadFile) suites.elementAt(index);
	}

	/**
	 * Installs a new suite. JAD name parameter may be a local filename
	 * as well as a HTTP URL. The corresponding JAR file is retrieved
	 * automatically.
	 */
	public void installSuite(String jadName) throws IOException {
		JadFile jadFile = new JadFile();

		/**
		 * Load the JAD file into a JadFile object.
		 */
		InputStream input = openInputStream(jadName);
		try {
			jadFile.load(input);
		}
		finally {
			input.close();
		}

		/**
		 * Load the JAR file into a temporary buffer.
		 */
		String jarName = getRealJarName(jadName, jadFile.getValue("MIDlet-Jar-Url"));
        
		input = openInputStream(jarName);
		ByteArrayOutputStream temp = new ByteArrayOutputStream();
		byte[] buffer = new byte[8192];

		try {
			int i = input.read(buffer);
			while (i != -1) {
				temp.write(buffer, 0, i);
				i = input.read(buffer);
			}
		}
		finally {
			input.close();
		}

		/**
		 * Check the JAR size.
		 */
		int jarSize = Integer.parseInt(jadFile.getValue("MIDlet-Jar-Size"));
		if (jarSize != temp.size()) {
			throw new IOException("JAR size doesn't match JAD file");
		}

		/**
		 * If we arrive here, everything is ok. So we add the new entry to the
		 * MIDlet database, or replace an existing suite, if it already exists.
		 */
		String name = new File(jadName).getName().toLowerCase();
		int dot = name.indexOf('.');
		if (dot != -1)
			name = name.substring(0, dot);

		String suitePath = path + File.separatorChar + name;
		String newJadName = suitePath + File.separatorChar + name + ".jad";
		String newJarName = suitePath + File.separatorChar + name + ".jar";

		int i = indexOfSuite(newJarName);
		if (i != -1)
			suites.removeElementAt(i);

		new File(suitePath).mkdirs();

		/**
		 * Write JAD file.
		 */
		jadFile.setValue("MIDlet-Jar-Url", newJarName);
		jadFile.save(newJadName);

		/**
		 * Write JAR file.
		 */
		FileOutputStream output = new FileOutputStream(newJarName);
		try {
			temp.writeTo(output);
			output.flush();
		}
		finally {
			output.close();
		}

		suites.addElement(jadFile);
	}

	/**
	 * Uninstalls a suite.
	 */
	public void removeSuite(int index) {
		JadFile jadFile = getSuite(index);

		/**
		 * Delete all files in the suite's directory, the the directory itself.
		 */
		String path = jadFile.getValue("MIDlet-Jar-Url");
		path = path.substring(0, path.lastIndexOf("\\"));
		File dir = new File(path);
		String[] files = dir.list();
		if (files != null) {
			for (int i = 0; i < files.length; i++) {
				new File(files[i]).delete();
			}
		}
		dir.delete();

		/**
		 * Throw away the JadFile object.
		 */
		suites.removeElementAt(index);
	}

	/**
	 * Checks existence of a suite by its JAR file path.
	 */
	private int indexOfSuite(String jarName) {
		for (int i = 0; i < getSuiteCount(); i++) {
			JadFile jadFile = getSuite(i);
			if (jadFile.getValue("MIDlet-Jar-Url").equals(jarName))
				return i;
		}

		return -1;
	}

	/**
	 * Opens an input stream for the given resource.
     */
	private InputStream openInputStream(String url) throws IOException {
		if (ApplicationManager.manager != null) {
			return ApplicationManager.manager.openInputStream(url);
		}
		else {
			String test = url.toLowerCase();
			if (test.startsWith("http://") || test.startsWith("file://")) {
				return new URL(url).openStream();
			}
			else {
				return new FileInputStream(url);
			}
		}
	}

    /**
     * Calculates a JAR file name from a JAD file name. Returns an absolute
     * pathname (relative to the JAD file's directory), if the JAR name is
     * not already absolute.
     */
    private String getRealJarName(String jadName, String jarName) {
        String test = jadName.toLowerCase();
        if (test.startsWith("http:") || test.startsWith("file:")) {
            String test2 = jarName.toLowerCase();
            if (test2.startsWith("http:") || test2.startsWith("file:")) {
                return jarName;
            }
            else {
                int slash = jadName.lastIndexOf('/');
                return jadName.substring(0, slash + 1) + jarName;
            }
        }
        else {
            if (new File(jarName).isAbsolute()) {
                return jarName;
            }
            else {
                return new File(jadName).getParent() + File.separatorChar + jarName;
            }
        }
    }
    
	/**
	 * Retrieves the titles of all installed suites as an array of strings.
	 */
	public String[] getSuiteList() {
		String[] result = new String[getSuiteCount()];

		for (int i = 0; i < result.length; i++) {
			result[i] = getSuite(i).getValue("MIDlet-Name");
		}

		return result;
	}

	/**
	 * Main method for testing purposes.
	 */
	public static void main(String[] args) {
        if (args.length == 0) {
            System.out.println("Parameters:");
            System.out.println("  <jam path> (install <jad file> | remove <int> | list)");
            System.exit(0);
        }
        
		try {
			MIDletDatabase db = new MIDletDatabase(args[0]);

			String command = args[1];

			if ("install".equals(command)) {
				db.installSuite(args[2]);
			}
			else if ("remove".equals(command)) {
				db.removeSuite(Integer.parseInt(args[2]));
			}
			else if ("list".equals(command)) {
				String[] suites = db.getSuiteList();

				for (int i = 0; i < suites.length; i++) {
					System.out.println(i + " - " + suites[i]);
				}
			}
			else {
				System.out.println("Illegal command: " + command);
			}
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
}
